use std::path::{Path, PathBuf};

use regex::Regex;
use ropey::RopeSlice;
use tower_lsp::lsp_types::*;

/// 根据 pos 获取到当前光标所在的字符串，相当于 getWordRangeAtPosition
pub fn get_definition_token(line: &RopeSlice, pos: Position) -> String {
    let mut token = String::new();
    let mut line_iter = line.chars();
    for _ in 0..(line.utf16_cu_to_char(pos.character as usize)) {
        line_iter.next();
    }
    let mut c = line_iter.prev();
    while c.is_some() && (c.unwrap().is_alphanumeric() || c.unwrap() == '_') {
        token.push(c.unwrap());
        c = line_iter.prev();
    }
    token = token.chars().rev().collect();
    line_iter = line.chars();
    for _ in 0..(line.utf16_cu_to_char(pos.character as usize)) {
        line_iter.next();
    }
    let mut c = line_iter.next();
    while c.is_some() && (c.unwrap().is_alphanumeric() || c.unwrap() == '_') {
        token.push(c.unwrap());
        c = line_iter.next();
    }
    token
}

pub fn get_word_range_at_position(line: &RopeSlice, pos: Position, regex: Regex) -> String {
    let mut token = String::new();
    let mut line_iter = line.chars();
    for _ in 0..(line.utf16_cu_to_char(pos.character as usize)) {
        line_iter.next();
    }
    let mut c = line_iter.prev();

    while c.is_some() && (regex.is_match(&c.unwrap().to_string())) {
        token.push(c.unwrap());
        c = line_iter.prev();
    }
    token = token.chars().rev().collect();
    line_iter = line.chars();
    for _ in 0..(line.utf16_cu_to_char(pos.character as usize)) {
        line_iter.next();
    }
    let mut c = line_iter.next();
    while c.is_some() && (regex.is_match(&c.unwrap().to_string())) {
        token.push(c.unwrap());
        c = line_iter.next();
    }
    token
}

/// 根据 uri 获取 hdl 的 language id
pub fn get_language_id_by_uri(uri: &Url) -> String {
    let path = uri.path();
    let ext_name = std::path::Path::new(path)
        .extension()
        .and_then(std::ffi::OsStr::to_str)
        .unwrap_or("");

    match ext_name {
        "vhd" | "vhdl" | "vho" | "vht" => "vhdl".to_string(),
        "v" | "V" | "vh" | "vl" => "verilog".to_string(),
        "sv" | "svh" => "systemverilog".to_string(),
        _ => "plaintext".to_string()
    }
}

/// 根据基础路径和给出的 path 路径，计算出绝对路径
/// 如果 path 是绝对路径直接返回
/// 会进行存在性检查，不存在的路径直接返回 None
/// 例子：
/// resolve_path("/home/ubuntu/hello.v", "./control.v") -> Some("/home/ubuntu/control.v")
/// resolve_path("/home/ubuntu/hello.v", "/opt/control.v") -> Some("/opt/control.v")
pub fn resolve_path(base: &str, path_string: &str) -> Option<PathBuf> {
    let path = Path::new(path_string);
    let abs_path: PathBuf;
    if path.is_absolute() {
        abs_path = PathBuf::from(path);
    } else {
        let base = Path::new(base).parent()?;
        abs_path = base.join(path);
    }
    if abs_path.exists() {
        let path = abs_path.as_path().to_owned();
        return Some(path);
    }
    None
}